# Argument

## Accessing Arguments

When you run a Bash script, you can pass arguments to it from the command line. These arguments can be accessed inside the script using special variables.  
Bash automatically assigns numbers to each argument:

* $0 - The script name
* $1 - First argument
* $2 - Second argument


Let’s explore an example of basic argument handling.  
Write this into `script.sh` :

```
#!/bin/bash
echo "Script name: $0"
echo "First argument: $1"
echo "Second argument: $2"
```
and execute it:

```bash
./script.sh apple macaron
```

??? example "Click to show the solution"
    ```bash
    Script name: ./script.sh
    First argument: apple
    Second argument: macaron
    ```

## Checking if an Argument is Provided

Before using an argument, it’s a good practice to check if it’s set.

```bash
#!/bin/bash
if [[ -z "$1" ]]; then
    echo "Error: No argument provided!"
    exit 1
fi

echo "Argument received: $1"
```

and execute it:

```bash
./script.sh
```

??? example "Click to show the solution"
    ```bash
    Error: No argument provided!
    ```

## Handling Multiple Arguments

You can use `$@` and `$#` to manage multiple arguments:
	
* $@ - All arguments as separate words
* $* - All arguments as a single string
* $# - Number of arguments provided

Let’s explore an example of looping through arguments.  
Write this into `script.sh` :

```bash
#!/bin/bash
echo "You provided $# arguments:"
for arg in "$@"; do
    echo "- $arg"
done
```

and execute it:

```bash
./script.sh apple macaron
```

??? example "Click to show the solution"
    ```bash
    You provided 2 arguments:
    - apple
    - macaron
    ```

## Named Arguments with Flags

To handle named arguments, use a `case` statement.

Let’s explore an example of parsing flags.  
Write this into `script.sh` :

```bash
#!/bin/bash

# Parse arguments using case
while [[ $# -gt 0 ]]; do
    case $1 in
        -f|--fruit)
            fruit="$2"
            shift 2 ;;  # remove flag & value
        -c|--cake)
            cake="$2"
            shift 2 ;;  # remove flag & value
        *)
            echo "Unknown option: $1"
            exit 1 ;;
    esac
done

echo "Fruit: $fruit"
echo "Cake: $cake"
```

and execute it:

```bash
./script.sh --fruit apple --cake macaron
```

??? example "Click to show the solution"
    ```bash
    Fruit: apple
    Cake: macaron
    ```

## Full Example

Below is a complete Bash script demonstrating how to handle arguments using a case statement. 

* It handle Named Arguments with Flags
* It validates arguments (one is an integer the other one a string)
* It checks all arguments are provided
* It provides a help


```bash
#!/bin/bash

# Initialize variables
num=""
text=""

# Parse arguments using case
while [[ $# -gt 0 ]]; do
    case "$1" in
        -n)  # Handle number argument
            
            if [[ "$2" =~ ^[0-9]+$ ]]; then
                num="$2"
            else
                echo "Error: -n requires an integer." >&2
                exit 1
            fi
            shift 2
            ;;
        -s)  # Handle string argument
            if [[ -n "$2" ]]; then
                text="$2"
            else
                echo "Error: -s requires a string." >&2
                exit 1
            fi
            shift 2
            ;;
        -h|--help)  # Handle help option
            echo "This script perform ...explanation here..."
            echo "Usage: $0 -n <integer> -s <string>"
            echo
            echo "Options:"
            echo "  -n <integer>  Provide an integer value."
            echo "  -s <string>   Provide a string value."
            echo "  -h, --help    Show this help message and exit."
            exit 0
            ;;
        *)  # Handle unknown options
            echo "Error: Unknown option $1" >&2
            echo "Usage: $0 -n <integer> -s <string>"
            ;;
    esac
done

# Check if required arguments are provided
if [[ -z "$num" || -z "$text" ]]; then
    echo "Error: Both -n <integer> and -s <string> are required." >&2
    show_help
fi

# Output the received values
echo "Integer provided: $num"
echo "String provided: $text"
```

Now try with several conditions:

```bash
./script.sh -n 42 -s "macaron"
```

??? example "Click to show the solution"
    ```bash
    Integer provided: 12
    String provided: macaron  
    ```


```bash
./script.sh -n apple -s macaron
```

??? example "Click to show the solution"
    ```bash
    Error: -n requires an integer.
    ```


```bash
./script.sh --help
```

??? example "Click to show the solution"
    ```bash
    Usage: ./script.sh -n <integer> -s <string>

    Options:
    -n <integer>  Provide an integer value.
    -s <string>   Provide a string value.
    -h, --help    Show this help message and exit.
    ```

!!! Success "Quick recap"
    In this section we've learned:

    - How to access arguments within a script
    - How to use named arguments
    - How to check provided arguments

